package org.itqop.chatit.utils;

import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.nio.charset.StandardCharsets;
import java.util.concurrent.CompletableFuture;
import java.util.regex.Pattern;

import org.slf4j.Logger;
import com.mojang.logging.LogUtils;

import com.google.gson.Gson;
import com.google.gson.JsonObject;

public class ProfanityChecker {

    private static final Gson GSON = new Gson();
    private static final HttpClient CLIENT = HttpClient.newHttpClient();
    private static final Logger LOGGER = LogUtils.getLogger();

    private static Pattern profanityPattern = null;

    public static CompletableFuture<Double> checkMessageAsync(String message) {
        return CompletableFuture.supplyAsync(() -> {
            try {
                String apiUrl = Config.hostApi;
                boolean useRegex = Config.useRegex;

                if (apiUrl == null || apiUrl.isEmpty()) {
                    if (useRegex) {
                        return containsProfanity(message) ? 1.0 : 0.0;
                    } else {
                        return 0.0;
                    }
                }

                String jsonRequest = GSON.toJson(new MessageRequest(message));
                HttpRequest request = HttpRequest.newBuilder()
                        .uri(URI.create(apiUrl))
                        .version(HttpClient.Version.HTTP_1_1)
                        .header("Content-Type", "application/json")
                        .POST(HttpRequest.BodyPublishers.ofString(jsonRequest, StandardCharsets.UTF_8))
                        .build();

                HttpResponse<String> response = CLIENT.send(request, HttpResponse.BodyHandlers.ofString());

                String responseBody = response.body();

                if (response.statusCode() != 200) {
                    throw new RuntimeException("Ошибка API: " + responseBody);
                }

                JsonObject jsonResponse = GSON.fromJson(responseBody, JsonObject.class);

                return jsonResponse.get("toxicity_score").getAsDouble();
            } catch (Exception e) {
                LOGGER.error(e.toString());
                if (Config.useRegex) {
                    return containsProfanity(message) ? 1.0 : 0.0;
                }
                return 0.0;
            }
        });
    }

    private static boolean containsProfanity(String message) {
        if (profanityPattern == null || !profanityPattern.pattern().equals(Config.profanityRegex)) {
            profanityPattern = Pattern.compile(Config.profanityRegex);
        }
        return profanityPattern.matcher(message).find();
    }

    private static class MessageRequest {
        private String text;

        public MessageRequest(String text) {
            this.text = text;
        }

        public String getText() {
            return text;
        }

        public void setText(String text) {
            this.text = text;
        }
    }
}
